@@ -64,7 +64,7 @@ class NTLMCredential(UniCredential):
 
 		}
 	
-	def build_context(self):
+	def build_context(self, *args, **kwargs):
 		if self.subprotocol.type == asyauthSubProtocol.NATIVE:
 			from asyauth.protocols.ntlm.client.native import NTLMClientNative
 			return NTLMClientNative(self)
@@ -80,6 +80,11 @@ class NTLMCredential(UniCredential):
 		elif self.subprotocol.type == asyauthSubProtocol.WSNET:
 			from asyauth.protocols.ntlm.client.wsnet import NTLMClientWSNET
 			return NTLMClientWSNET(self)
+		
+		elif self.subprotocol.type == asyauthSubProtocol.WSNETDIRECT:
+			from asyauth.protocols.ntlm.client.wsnetdirect import NTLMClientWSNETDirect
+			return NTLMClientWSNETDirect(self)
+		
 		elif self.subprotocol.type == asyauthSubProtocol.CUSTOM:
 			return self.subprotocol.factoryobj.build_context(self)
 		else:
