@@ -1,18 +1,20 @@
 
-from asyauth import logger
+import datetime
+
+from asyauth.protocols.kerberos import logger
 from asyauth.common.winapi.constants import ISC_REQ
-from asyauth.protocols.kerberos.gssapi import get_gssapi
+from asyauth.common.credentials.kerberos import KerberosCredential
+from asyauth.protocols.kerberos.gssapi import get_gssapi, KRB5_MECH_INDEP_TOKEN
 from asyauth.protocols.kerberos.gssapismb import get_gssapi as gssapi_smb
-from minikerberos.protocol.asn1_structs import AP_REP, EncAPRepPart, Ticket
-from minikerberos.protocol.structures import ChecksumFlags
+
+from minikerberos.common.spn import KerberosSPN
 from minikerberos.gssapi.gssapi import GSSAPIFlags
+from minikerberos.protocol.asn1_structs import AP_REP, EncAPRepPart, Ticket, EncryptedData
+from minikerberos.protocol.constants import MESSAGE_TYPE
+from minikerberos.protocol.ticketutils import construct_apreq_from_ticket
 from minikerberos.protocol.encryption import Key, _enctype_table
 from minikerberos.aioclient import AIOKerberosClient
-from asyauth.common.credentials.kerberos import KerberosCredential
-from minikerberos.common.spn import KerberosSPN
-import datetime
-from minikerberos.protocol.constants import MESSAGE_TYPE
-from minikerberos.protocol.asn1_structs import AP_REP, EncAPRepPart, EncryptedData, Ticket
+
 
 
 class KerberosClientNative:
@@ -53,20 +55,20 @@ class KerberosClientNative:
 		"""
 		return GSSAPIFlags.GSS_C_CONF_FLAG in self.flags
 				
-	async def sign(self, data, message_no, direction = 'init'):
+	async def sign(self, data:bytes, message_no:int, direction = 'init'):
 		"""
 		Signs a message. 
 		"""
 		return self.gssapi.GSS_GetMIC(data, message_no, direction = direction)	
 		
-	async def encrypt(self, data, message_no, *args, **kwargs):
+	async def encrypt(self, data:bytes, message_no:int, *args, **kwargs):
 		"""
 		Encrypts a message. 
 		"""
 		data, eeee  = self.gssapi.GSS_Wrap(data, message_no, *args, **kwargs)
 		return data, eeee 
 		
-	async def decrypt(self, data, message_no, *args, **kwargs):
+	async def decrypt(self, data:bytes, message_no:int, *args, **kwargs):
 		"""
 		Decrypts message. Also performs integrity checking.
 		"""
@@ -76,7 +78,7 @@ class KerberosClientNative:
 	def get_session_key(self):
 		return self.session_key.contents
 
-	def iscreq_to_gssapiflags(self, flags):
+	def iscreq_to_gssapiflags(self, flags:ISC_REQ):
 		if flags is None:
 			return self.flags
 		kflags = GSSAPIFlags.GSS_C_CONF_FLAG |\
@@ -110,18 +112,21 @@ class KerberosClientNative:
 		return kflags
 		
 	
-	async def authenticate(self, authData, flags = None, seq_number = 0, cb_data = None, spn = None):
+	async def authenticate(self, authData:bytes, flags:ISC_REQ = None, seq_number:int = 0, cb_data:bytes = None, spn:str = None):
 		"""
 		This function is called (multiple times depending on the flags) to perform authentication. 
 		"""
 		try:
 			self.flags = self.iscreq_to_gssapiflags(flags)
+			logger.debug('Flags: %s' % self.flags)
+			
 
 			if spn is None:
 				raise Exception("SPN is needed for kerberos!")
 			else:
-				spn = KerberosSPN.from_target_string(spn)
+				spn = KerberosSPN.from_spn(spn)
 
+			logger.debug('SPN: %s' % spn)
 			if self.kc is None:
 				self.kc = AIOKerberosClient(self.ccred, self.credential.target)
 
@@ -135,12 +140,24 @@ class KerberosClientNative:
 						# just printing this to debug...
 						logger.debug('CCACHE SPN record: %s' % target)
 					tgs, encpart, self.session_key = await self.kc.get_TGS(spn)
+					logger.debug('Got TGS from CCACHE!')
 					
 					self.from_ccache = True
 				except:
+					# fetching TGT
 					tgt = await self.kc.get_TGT(override_etype = self.credential.etypes)
+					# if the target server is in a different domain, we need to get a referral ticket
+					if self.credential.cross_target is not None:
+						# cross-domain kerberos
+						ref_tgs, ref_encpart, ref_key, new_factory = await self.kc.get_referral_ticket(self.credential.cross_realm, self.credential.cross_target.get_ip_or_hostname())
+						self.kc = new_factory.get_client()
+						spn.domain = self.credential.cross_realm
 					tgs, encpart, self.session_key = await self.kc.get_TGS(spn)#, override_etype = self.preferred_etypes)
 				
+				logger.debug('TGS: %s' % tgs)
+				logger.debug('encpart: %s' % encpart)
+				logger.debug('session_key: %s' % self.session_key)
+
 				ap_opts = []
 				if GSSAPIFlags.GSS_C_MUTUAL_FLAG in self.flags or GSSAPIFlags.GSS_C_DCE_STYLE in self.flags:
 					if GSSAPIFlags.GSS_C_MUTUAL_FLAG in self.flags:
@@ -156,15 +173,18 @@ class KerberosClientNative:
 							cb_data = cb_data
 						)
 					else:
-						apreq = self.kc.construct_apreq_from_ticket(
+						apreq = construct_apreq_from_ticket(
 							Ticket(tgs['ticket']).dump(), 
 							self.session_key, 
-							tgs['crealm'], tgs['cname']['name-string'][0], 
+							tgs['crealm'], 
+							tgs['cname']['name-string'][0], 
 							flags = self.flags, 
 							seq_number = self.seq_number, 
 							ap_opts = ap_opts, 
 							cb_data = cb_data
 						)
+					
+					logger.debug('APREQ constructed: %s' % apreq)
 					return apreq, True, None
 				
 				else:
@@ -179,7 +199,7 @@ class KerberosClientNative:
 							ap_opts=ap_opts, 
 							cb_data = cb_data)
 					else:
-						apreq = self.kc.construct_apreq_from_ticket(
+						apreq = construct_apreq_from_ticket(
 							Ticket(tgs['ticket']).dump(), 
 							self.session_key, 
 							tgs['crealm'], 
@@ -190,18 +210,25 @@ class KerberosClientNative:
 							cb_data = cb_data
 						)
 					
-					
+					logger.debug('APREQ constructed: %s' % apreq)
 					self.gssapi = get_gssapi(self.session_key)
 					return apreq, False, None
 
 			else:
 				self.seq_number = seq_number
 
-				aprep = AP_REP.load(authData).native
+				logger.debug('Processing AP_REP %s' % authData.hex())
+				try:
+					temp = KRB5_MECH_INDEP_TOKEN.from_bytes(authData)
+					aprep = AP_REP.load(temp.data[2:]).native
+				except Exception as e:
+					aprep = AP_REP.load(authData).native
+				
+				logger.debug('AP_REP: %s' % aprep)
 				cipher = _enctype_table[int(aprep['enc-part']['etype'])]()
 				cipher_text = aprep['enc-part']['cipher']
 				temp = cipher.decrypt(self.session_key, 12, cipher_text)
-					
+				
 				enc_part = EncAPRepPart.load(temp).native
 				cipher = _enctype_table[int(enc_part['subkey']['keytype'])]()
 					
@@ -212,17 +239,21 @@ class KerberosClientNative:
 				apreppart_data['seq-number'] = enc_part['seq-number']
 				#print('seq %s' % enc_part['seq-number'])
 				#self.seq_number = 0 #enc_part['seq-number']
-					
+				
+				logger.debug('apreppart_data: %s' % apreppart_data)
 				apreppart_data_enc = cipher.encrypt(self.session_key, 12, EncAPRepPart(apreppart_data).dump(), None)
 					
 				#overriding current session key
 				self.session_key = Key(cipher.enctype, enc_part['subkey']['keyvalue'])
-					
+				
+				logger.debug('SessionKey: %s' % self.session_key)
+
 				ap_rep = {}
 				ap_rep['pvno'] = 5 
 				ap_rep['msg-type'] = MESSAGE_TYPE.KRB_AP_REP.value
 				ap_rep['enc-part'] = EncryptedData({'etype': self.session_key.enctype, 'cipher': apreppart_data_enc}) 
-					
+				
+				logger.debug('AP_REP: %s' % ap_rep)
 				token = AP_REP(ap_rep).dump()
 				if GSSAPIFlags.GSS_C_DCE_STYLE in self.flags:
 					self.gssapi = gssapi_smb(self.session_key)
